import * as i0 from '@angular/core';
import { InjectionToken, forwardRef, Directive, Attribute, inject, Inject, Output, Self, Input, EventEmitter, SkipSelf, ContentChildren, Injectable, Pipe, ElementRef, NgModule } from '@angular/core';
import { interval, animationFrameScheduler, Subject, of } from 'rxjs';
import { mapTo, tap, map, distinctUntilChanged, skipWhile, debounceTime, filter, share, switchMap } from 'rxjs/operators';
import { __decorate } from 'tslib';

function processAudioParam(param, value, currentTime = 0) {
    if (param.cancelAndHoldAtTime) {
        param.cancelAndHoldAtTime(currentTime);
    }
    else {
        param.cancelScheduledValues(currentTime);
        param.setValueAtTime(guard(param.value), currentTime);
    }
    if (typeof value === 'number') {
        param.setValueAtTime(guard(value), currentTime);
        return;
    }
    if (value instanceof Array) {
        processSchedule(param, value, currentTime);
        return;
    }
    if (!('mode' in value)) {
        param.setValueCurveAtTime(value.value, currentTime, value.duration);
        return;
    }
    param.setValueAtTime(guard(param.value), currentTime);
    processAutomation(param, value, currentTime);
}
function processSchedule(param, value, currentTime) {
    value.forEach(automation => {
        if ('mode' in automation) {
            processAutomation(param, automation, currentTime);
        }
        else {
            param.setValueCurveAtTime(automation.value, currentTime, automation.duration);
        }
        currentTime += automation.duration;
    });
}
function processAutomation(param, { value, mode = 'instant', duration }, currentTime) {
    switch (mode) {
        case 'instant':
            param.setValueAtTime(guard(value), currentTime);
            param.setValueAtTime(guard(value), currentTime + duration);
            break;
        case 'exponential':
            if (value < 0 || value * param.value < 0) {
                param.linearRampToValueAtTime(guard(value), currentTime + duration);
            }
            else {
                param.exponentialRampToValueAtTime(guard(value), currentTime + duration);
            }
            param.setValueAtTime(guard(value), currentTime + duration);
            break;
        case 'linear':
            param.linearRampToValueAtTime(guard(value), currentTime + duration);
            break;
    }
}
function guard(v) {
    return v || 0.00000001;
}

function audioParam(param = '') {
    const decorator = (target, propertyKey) => {
        Object.defineProperty(target, propertyKey, {
            set(value) {
                value = typeof value === 'string' ? Number.parseFloat(value) : value;
                const audioParam = this instanceof AudioWorkletNode
                    ? this.parameters.get(propertyKey)
                    : this[param];
                if (audioParam instanceof AudioParam) {
                    processAudioParam(audioParam, value, this.context.currentTime);
                }
                else {
                    // Fallback for older browsers
                    Object.defineProperty(target, propertyKey, { value });
                }
            },
        });
    };
    return decorator;
}

const AUDIO_CONTEXT = new InjectionToken('Web Audio API context', {
    providedIn: 'root',
    factory: () => new AudioContext(),
});

function latencyHintFactory(latencyHint) {
    return latencyHint === null
        ? undefined
        : Number.parseFloat(latencyHint) || latencyHint;
}

class WebAudioContext extends AudioContext {
    constructor(latencyHint, sampleRate) {
        super({
            latencyHint: latencyHintFactory(latencyHint),
            sampleRate: parseInt(sampleRate || '', 10) || undefined,
        });
    }
    ngOnDestroy() {
        this.close();
    }
}
WebAudioContext.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioContext, deps: [{ token: 'latencyHint', attribute: true }, { token: 'sampleRate', attribute: true }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioContext.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioContext, selector: "[waAudioContext]", providers: [
        {
            provide: AUDIO_CONTEXT,
            useExisting: forwardRef(() => WebAudioContext),
        },
    ], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioContext, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waAudioContext]',
                    providers: [
                        {
                            provide: AUDIO_CONTEXT,
                            useExisting: forwardRef(() => WebAudioContext),
                        },
                    ],
                }]
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Attribute,
                    args: ['latencyHint']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['sampleRate']
                }] }]; } });

/**
 * This is mostly for internal use only
 */
const CONSTRUCTOR_SUPPORT = new InjectionToken('Tests if constructor mode of node creation is supported or a fallback to factory method is needed', {
    providedIn: 'root',
    factory: () => {
        try {
            return !!new GainNode(inject(AUDIO_CONTEXT));
        }
        catch (_) {
            return false;
        }
    },
});

class WebAudioChannel extends GainNode {
    constructor(context, modern) {
        if (modern) {
            super(context);
        }
        else {
            const result = context.createGain();
            Object.setPrototypeOf(result, WebAudioChannel.prototype);
            return result;
        }
    }
    ngOnDestroy() {
        this.disconnect();
    }
}
WebAudioChannel.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioChannel, deps: [{ token: AUDIO_CONTEXT }, { token: CONSTRUCTOR_SUPPORT }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioChannel.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioChannel, selector: "[waChannel]", exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioChannel, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waChannel]',
                    exportAs: 'AudioNode',
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }]; } });

const POLLING_TIME = 100;

const AUDIO_NODE = new InjectionToken('Web Audio API audio node', {
    factory: () => null,
});
function asAudioNode(useExisting) {
    return {
        provide: AUDIO_NODE,
        useExisting,
    };
}

function connect(source, destination) {
    if (source && destination) {
        // @ts-ignore TS does not have a union override for connect method
        source.connect(destination);
    }
}

class WebAudioDestination extends AnalyserNode {
    constructor(context, node, modern) {
        if (modern) {
            super(context);
            WebAudioDestination.init(this, node);
        }
        else {
            const result = context.createAnalyser();
            Object.setPrototypeOf(result, WebAudioDestination.prototype);
            WebAudioDestination.init(result, node);
            return result;
        }
    }
    ngOnDestroy() {
        this.disconnect();
    }
    isSilent(array) {
        return Math.abs(array.reduce((acc, cur) => acc + cur, 0)) < 0.001;
    }
    static init(that, node) {
        connect(node, that);
        that.fftSize = 256;
        that.connect(that.context.destination);
        that.quiet = interval(POLLING_TIME).pipe(mapTo(new Float32Array(that.fftSize)), tap(array => that.getFloatTimeDomainData(array)), map(array => that.isSilent(array)), distinctUntilChanged(), skipWhile(isSilent => isSilent), debounceTime(5000), filter(isSilent => isSilent));
    }
}
WebAudioDestination.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioDestination, deps: [{ token: AUDIO_CONTEXT }, { token: AUDIO_NODE }, { token: CONSTRUCTOR_SUPPORT }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioDestination.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioDestination, selector: "[waAudioDestinationNode]", outputs: { quiet: "quiet" }, exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioDestination, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waAudioDestinationNode]',
                    exportAs: 'AudioNode',
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: AudioNode, decorators: [{
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }]; }, propDecorators: { quiet: [{
                type: Output
            }] } });

function fallbackAudioParam(value) {
    if (!value) {
        return 0;
    }
    if (typeof value === 'number') {
        return value;
    }
    if (value instanceof Array) {
        const last = value[value.length - 1].value;
        return typeof last === 'number' ? last : last[last.length - 1];
    }
    if (value.value instanceof Array) {
        return value.value[value.value.length - 1];
    }
    return value.value;
}

class WebAudioListener extends GainNode {
    constructor(context, modern) {
        if (modern) {
            super(context);
        }
        else {
            const result = context.createGain();
            Object.setPrototypeOf(result, WebAudioListener.prototype);
            return result;
        }
    }
    get forwardX() {
        return this.context.listener.forwardX;
    }
    get forwardY() {
        return this.context.listener.forwardY;
    }
    get forwardZ() {
        return this.context.listener.forwardZ;
    }
    get positionX() {
        return this.context.listener.positionX;
    }
    get positionY() {
        return this.context.listener.positionY;
    }
    get positionZ() {
        return this.context.listener.positionZ;
    }
    get upX() {
        return this.context.listener.upX;
    }
    get upY() {
        return this.context.listener.upY;
    }
    get upZ() {
        return this.context.listener.upZ;
    }
    ngOnChanges() {
        if (this.context.listener.positionX instanceof AudioParam) {
            return;
        }
        // Fallback for older browsers
        this.context.listener.setOrientation(fallbackAudioParam(this.forwardXParam), fallbackAudioParam(this.forwardYParam), fallbackAudioParam(this.forwardZParam), fallbackAudioParam(this.upXParam), fallbackAudioParam(this.upYParam), fallbackAudioParam(this.upZParam));
        this.context.listener.setPosition(fallbackAudioParam(this.positionXParam), fallbackAudioParam(this.positionYParam), fallbackAudioParam(this.positionZParam));
    }
}
WebAudioListener.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioListener, deps: [{ token: AUDIO_CONTEXT, self: true }, { token: CONSTRUCTOR_SUPPORT }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioListener.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioListener, selector: "[waAudioContext],[waOfflineAudioContext][length][sampleRate]", inputs: { forwardXParam: ["forwardX", "forwardXParam"], forwardYParam: ["forwardY", "forwardYParam"], forwardZParam: ["forwardZ", "forwardZParam"], positionXParam: ["positionX", "positionXParam"], positionYParam: ["positionY", "positionYParam"], positionZParam: ["positionZ", "positionZParam"], upXParam: ["upX", "upXParam"], upYParam: ["upY", "upYParam"], upZParam: ["upZ", "upZParam"] }, usesInheritance: true, usesOnChanges: true, ngImport: i0 });
__decorate([
    audioParam('forwardX')
], WebAudioListener.prototype, "forwardXParam", void 0);
__decorate([
    audioParam('forwardY')
], WebAudioListener.prototype, "forwardYParam", void 0);
__decorate([
    audioParam('forwardZ')
], WebAudioListener.prototype, "forwardZParam", void 0);
__decorate([
    audioParam('positionX')
], WebAudioListener.prototype, "positionXParam", void 0);
__decorate([
    audioParam('positionY')
], WebAudioListener.prototype, "positionYParam", void 0);
__decorate([
    audioParam('positionZ')
], WebAudioListener.prototype, "positionZParam", void 0);
__decorate([
    audioParam('upX')
], WebAudioListener.prototype, "upXParam", void 0);
__decorate([
    audioParam('upY')
], WebAudioListener.prototype, "upYParam", void 0);
__decorate([
    audioParam('upZ')
], WebAudioListener.prototype, "upZParam", void 0);
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioListener, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waAudioContext],[waOfflineAudioContext][length][sampleRate]',
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Self
                }, {
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }]; }, propDecorators: { forwardXParam: [{
                type: Input,
                args: ['forwardX']
            }], forwardYParam: [{
                type: Input,
                args: ['forwardY']
            }], forwardZParam: [{
                type: Input,
                args: ['forwardZ']
            }], positionXParam: [{
                type: Input,
                args: ['positionX']
            }], positionYParam: [{
                type: Input,
                args: ['positionY']
            }], positionZParam: [{
                type: Input,
                args: ['positionZ']
            }], upXParam: [{
                type: Input,
                args: ['upX']
            }], upYParam: [{
                type: Input,
                args: ['upY']
            }], upZParam: [{
                type: Input,
                args: ['upZ']
            }] } });

class WebAudioOfflineContext extends OfflineAudioContext {
    constructor(length, sampleRate, numberOfChannels, autoplay) {
        super(parseInt(numberOfChannels || '', 10) || 1, parseInt(length, 10), parseInt(sampleRate, 10));
        this.complete = new EventEmitter();
        this.oncomplete = ({ renderedBuffer }) => this.complete.emit(renderedBuffer);
        if (autoplay !== null) {
            this.startRendering();
        }
    }
}
WebAudioOfflineContext.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioOfflineContext, deps: [{ token: 'length', attribute: true }, { token: 'sampleRate', attribute: true }, { token: 'numberOfChannels', attribute: true }, { token: 'autoplay', attribute: true }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioOfflineContext.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioOfflineContext, selector: "[waOfflineAudioContext][length][sampleRate]", outputs: { complete: "complete" }, providers: [
        {
            provide: AUDIO_CONTEXT,
            useExisting: forwardRef(() => WebAudioOfflineContext),
        },
    ], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioOfflineContext, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waOfflineAudioContext][length][sampleRate]',
                    providers: [
                        {
                            provide: AUDIO_CONTEXT,
                            useExisting: forwardRef(() => WebAudioOfflineContext),
                        },
                    ],
                }]
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Attribute,
                    args: ['length']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['sampleRate']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['numberOfChannels']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['autoplay']
                }] }]; }, propDecorators: { complete: [{
                type: Output
            }] } });

class WebAudioOutput extends GainNode {
    constructor(context, node, modern) {
        if (modern) {
            super(context);
            connect(node, this);
        }
        else {
            const result = context.createGain();
            Object.setPrototypeOf(result, WebAudioOutput.prototype);
            connect(node, result);
            return result;
        }
    }
    set waOutput(destination) {
        this.disconnect();
        connect(this, destination);
    }
    ngOnDestroy() {
        this.disconnect();
    }
    static init(that, node) {
        connect(node, that);
    }
}
WebAudioOutput.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioOutput, deps: [{ token: AUDIO_CONTEXT }, { token: AUDIO_NODE }, { token: CONSTRUCTOR_SUPPORT }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioOutput.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioOutput, selector: "[waOutput]", inputs: { waOutput: "waOutput" }, usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioOutput, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waOutput]',
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: AudioNode, decorators: [{
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }]; }, propDecorators: { waOutput: [{
                type: Input
            }] } });

class WebAudioMediaStreamDestination extends MediaStreamAudioDestinationNode {
    constructor(context, node, modern) {
        if (modern) {
            super(context);
            connect(node, this);
        }
        else {
            const result = context.createMediaStreamDestination();
            Object.setPrototypeOf(result, WebAudioMediaStreamDestination.prototype);
            connect(node, result);
            return result;
        }
    }
}
WebAudioMediaStreamDestination.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioMediaStreamDestination, deps: [{ token: AUDIO_CONTEXT }, { token: AUDIO_NODE }, { token: CONSTRUCTOR_SUPPORT }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioMediaStreamDestination.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioMediaStreamDestination, selector: "[waMediaStreamAudioDestinationNode]", exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioMediaStreamDestination, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waMediaStreamAudioDestinationNode]',
                    exportAs: 'AudioNode',
                }]
        }], ctorParameters: function () { return [{ type: AudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: AudioNode, decorators: [{
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }]; } });

function parse(value, fallback) {
    const parsed = parseFloat(value || '');
    return isNaN(parsed) ? fallback : parsed;
}

class WebAudioAnalyser extends AnalyserNode {
    constructor(context, node, modern, fftSizeArg, maxDecibelsArg, minDecibelsArg, smoothingTimeConstantArg) {
        const fftSize = parse(fftSizeArg, 2048);
        const maxDecibels = parse(maxDecibelsArg, -30);
        const minDecibels = parse(minDecibelsArg, -100);
        const smoothingTimeConstant = parse(smoothingTimeConstantArg, 0.8);
        if (modern) {
            super(context, { fftSize, maxDecibels, minDecibels, smoothingTimeConstant });
            WebAudioAnalyser.init(this, node);
        }
        else {
            const result = context.createAnalyser();
            Object.setPrototypeOf(result, WebAudioAnalyser.prototype);
            WebAudioAnalyser.init(result, node);
            result.fftSize = fftSize;
            result.maxDecibels = maxDecibels;
            result.minDecibels = minDecibels;
            result.smoothingTimeConstant = smoothingTimeConstant;
            return result;
        }
    }
    ngOnDestroy() {
        this.disconnect();
    }
    static init(that, node) {
        connect(node, that);
        let freqByte = new Uint8Array(that.frequencyBinCount);
        let freqFloat = new Float32Array(that.frequencyBinCount);
        let timeByte = new Uint8Array(that.fftSize);
        let timeFloat = new Float32Array(that.fftSize);
        that.frequencyByte$ = interval(0, animationFrameScheduler).pipe(map(() => {
            if (freqByte.length !== that.frequencyBinCount) {
                freqByte = new Uint8Array(that.frequencyBinCount);
            }
            that.getByteFrequencyData(freqByte);
            return freqByte;
        }), share());
        that.frequencyFloat$ = interval(0, animationFrameScheduler).pipe(map(() => {
            if (freqFloat.length !== that.frequencyBinCount) {
                freqFloat = new Float32Array(that.frequencyBinCount);
            }
            that.getFloatFrequencyData(freqFloat);
            return freqFloat;
        }), share());
        that.timeByte$ = interval(0, animationFrameScheduler).pipe(map(() => {
            if (timeByte.length !== that.fftSize) {
                timeByte = new Uint8Array(that.frequencyBinCount);
            }
            that.getByteTimeDomainData(timeByte);
            return timeByte;
        }), share());
        that.timeFloat$ = interval(0, animationFrameScheduler).pipe(map(() => {
            if (timeFloat.length !== that.fftSize) {
                timeFloat = new Float32Array(that.frequencyBinCount);
            }
            that.getFloatTimeDomainData(timeFloat);
            return timeFloat;
        }), share());
    }
}
WebAudioAnalyser.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioAnalyser, deps: [{ token: AUDIO_CONTEXT }, { token: AUDIO_NODE, skipSelf: true }, { token: CONSTRUCTOR_SUPPORT }, { token: 'fftSize', attribute: true }, { token: 'maxDecibels', attribute: true }, { token: 'minDecibels', attribute: true }, { token: 'smoothingTimeConstant', attribute: true }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioAnalyser.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioAnalyser, selector: "[waAnalyserNode]", inputs: { fftSize: "fftSize", minDecibels: "minDecibels", maxDecibels: "maxDecibels", smoothingTimeConstant: "smoothingTimeConstant", channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation" }, outputs: { frequencyByte$: "frequencyByte$", frequencyFloat$: "frequencyFloat$", timeByte$: "timeByte$", timeFloat$: "timeFloat$" }, providers: [asAudioNode(WebAudioAnalyser)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioAnalyser, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waAnalyserNode]',
                    exportAs: 'AudioNode',
                    inputs: [
                        'fftSize',
                        'minDecibels',
                        'maxDecibels',
                        'smoothingTimeConstant',
                        'channelCount',
                        'channelCountMode',
                        'channelInterpretation',
                    ],
                    providers: [asAudioNode(WebAudioAnalyser)],
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: AudioNode, decorators: [{
                    type: SkipSelf
                }, {
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['fftSize']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['maxDecibels']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['minDecibels']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['smoothingTimeConstant']
                }] }]; }, propDecorators: { frequencyByte$: [{
                type: Output
            }], frequencyFloat$: [{
                type: Output
            }], timeByte$: [{
                type: Output
            }], timeFloat$: [{
                type: Output
            }] } });

class WebAudioBiquadFilter extends BiquadFilterNode {
    constructor(context, modern, node, detuneArg, frequencyArg, gainArg, QArg) {
        const detune = parse(detuneArg, 0);
        const frequency = parse(frequencyArg, 350);
        const gain = parse(gainArg, 0);
        const Q = parse(QArg, 1);
        if (modern) {
            super(context, { detune, frequency, gain, Q });
            connect(node, this);
        }
        else {
            const result = context.createBiquadFilter();
            Object.setPrototypeOf(result, WebAudioBiquadFilter.prototype);
            result.detune.value = detune;
            result.frequency.value = frequency;
            result.gain.value = gain;
            result.Q.value = Q;
            connect(node, result);
            return result;
        }
    }
    ngOnDestroy() {
        this.disconnect();
    }
    static init(that, node) {
        connect(node, that);
    }
}
WebAudioBiquadFilter.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioBiquadFilter, deps: [{ token: AUDIO_CONTEXT }, { token: CONSTRUCTOR_SUPPORT }, { token: AUDIO_NODE, skipSelf: true }, { token: 'detune', attribute: true }, { token: 'frequency', attribute: true }, { token: 'gain', attribute: true }, { token: 'Q', attribute: true }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioBiquadFilter.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioBiquadFilter, selector: "[waBiquadFilterNode]", inputs: { type: "type", channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation", detuneParam: ["detune", "detuneParam"], frequencyParam: ["frequency", "frequencyParam"], gainParam: ["gain", "gainParam"], qParam: ["Q", "qParam"] }, providers: [asAudioNode(WebAudioBiquadFilter)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
__decorate([
    audioParam('detune')
], WebAudioBiquadFilter.prototype, "detuneParam", void 0);
__decorate([
    audioParam('frequency')
], WebAudioBiquadFilter.prototype, "frequencyParam", void 0);
__decorate([
    audioParam('gain')
], WebAudioBiquadFilter.prototype, "gainParam", void 0);
__decorate([
    audioParam('Q')
], WebAudioBiquadFilter.prototype, "qParam", void 0);
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioBiquadFilter, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waBiquadFilterNode]',
                    exportAs: 'AudioNode',
                    inputs: ['type', 'channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [asAudioNode(WebAudioBiquadFilter)],
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }, { type: AudioNode, decorators: [{
                    type: SkipSelf
                }, {
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['detune']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['frequency']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['gain']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['Q']
                }] }]; }, propDecorators: { detuneParam: [{
                type: Input,
                args: ['detune']
            }], frequencyParam: [{
                type: Input,
                args: ['frequency']
            }], gainParam: [{
                type: Input,
                args: ['gain']
            }], qParam: [{
                type: Input,
                args: ['Q']
            }] } });

class WebAudioChannelMerger extends ChannelMergerNode {
    constructor(inputs, context, modern) {
        const numberOfInputs = parseInt(inputs || '', 10) || 6;
        if (modern) {
            super(context, { numberOfInputs });
        }
        else {
            const result = context.createChannelMerger(numberOfInputs);
            Object.setPrototypeOf(result, WebAudioChannelMerger.prototype);
            return result;
        }
    }
    set channels(channels) {
        channels.forEach((node, index) => {
            node.connect(this, 0, index);
        });
    }
    ngOnDestroy() {
        this.disconnect();
    }
}
WebAudioChannelMerger.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioChannelMerger, deps: [{ token: 'numberOfInputs', attribute: true }, { token: AUDIO_CONTEXT }, { token: CONSTRUCTOR_SUPPORT }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioChannelMerger.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioChannelMerger, selector: "[waChannelMergerNode]", inputs: { channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation" }, providers: [asAudioNode(WebAudioChannelMerger)], queries: [{ propertyName: "channels", predicate: WebAudioChannel }], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioChannelMerger, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waChannelMergerNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [asAudioNode(WebAudioChannelMerger)],
                }]
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Attribute,
                    args: ['numberOfInputs']
                }] }, { type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }]; }, propDecorators: { channels: [{
                type: ContentChildren,
                args: [WebAudioChannel, { descendants: false }]
            }] } });

class WebAudioChannelSplitter extends ChannelSplitterNode {
    constructor(outputs, context, node, modern) {
        const numberOfOutputs = parseInt(outputs || '', 10) || 6;
        if (modern) {
            super(context, { numberOfOutputs });
            connect(node, this);
        }
        else {
            const result = context.createChannelSplitter(numberOfOutputs);
            Object.setPrototypeOf(result, WebAudioChannelSplitter.prototype);
            connect(node, result);
            return result;
        }
    }
    set channels(channels) {
        this.disconnect();
        channels
            .filter(node => !!node)
            .forEach((node, index) => {
            this.connect(node, index);
        });
    }
    ngOnDestroy() {
        this.disconnect();
    }
}
WebAudioChannelSplitter.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioChannelSplitter, deps: [{ token: 'numberOfOutputs', attribute: true }, { token: AUDIO_CONTEXT }, { token: AUDIO_NODE, skipSelf: true }, { token: CONSTRUCTOR_SUPPORT }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioChannelSplitter.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioChannelSplitter, selector: "[waChannelSplitterNode]", inputs: { channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation" }, providers: [
        {
            provide: AUDIO_NODE,
            useValue: null,
        },
    ], queries: [{ propertyName: "channels", predicate: AUDIO_NODE }], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioChannelSplitter, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waChannelSplitterNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useValue: null,
                        },
                    ],
                }]
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Attribute,
                    args: ['numberOfOutputs']
                }] }, { type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: AudioNode, decorators: [{
                    type: SkipSelf
                }, {
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }]; }, propDecorators: { channels: [{
                type: ContentChildren,
                args: [AUDIO_NODE, { descendants: false }]
            }] } });

class AudioBufferService {
    constructor(context) {
        this.context = context;
        this.cache = new Map();
    }
    fetch(url) {
        return new Promise((resolve, reject) => {
            if (this.cache.has(url)) {
                resolve(this.cache.get(url));
                return;
            }
            const request = new XMLHttpRequest();
            request.open('GET', url, true);
            request.responseType = 'arraybuffer';
            request.onerror = reject;
            request.onabort = reject;
            request.onload = () => {
                this.context.decodeAudioData(request.response, buffer => {
                    this.cache.set(url, buffer);
                    resolve(buffer);
                }, reject);
            };
            request.send();
        });
    }
}
AudioBufferService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: AudioBufferService, deps: [{ token: AUDIO_CONTEXT }], target: i0.ɵɵFactoryTarget.Injectable });
AudioBufferService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: AudioBufferService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: AudioBufferService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root',
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }]; } });

class WebAudioConvolver extends ConvolverNode {
    constructor(audioBufferService, context, node, modern) {
        if (modern) {
            super(context);
            WebAudioConvolver.init(this, node, audioBufferService);
        }
        else {
            const result = context.createConvolver();
            Object.setPrototypeOf(result, WebAudioConvolver.prototype);
            WebAudioConvolver.init(result, node, audioBufferService);
            return result;
        }
    }
    set bufferSetter(source) {
        this.buffer$.next(source);
    }
    ngOnDestroy() {
        this.buffer$.complete();
        this.disconnect();
    }
    static init(that, node, audioBufferService) {
        connect(node, that);
        that.buffer$ = new Subject();
        that.buffer$
            .pipe(switchMap(source => typeof source === 'string'
            ? audioBufferService.fetch(source)
            : of(source)))
            .subscribe(buffer => {
            that.buffer = buffer;
        });
    }
}
WebAudioConvolver.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioConvolver, deps: [{ token: AudioBufferService }, { token: AUDIO_CONTEXT }, { token: AUDIO_NODE, skipSelf: true }, { token: CONSTRUCTOR_SUPPORT }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioConvolver.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioConvolver, selector: "[waConvolverNode]", inputs: { normalize: "normalize", channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation", bufferSetter: ["buffer", "bufferSetter"] }, providers: [asAudioNode(WebAudioConvolver)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioConvolver, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waConvolverNode]',
                    exportAs: 'AudioNode',
                    inputs: ['normalize', 'channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [asAudioNode(WebAudioConvolver)],
                }]
        }], ctorParameters: function () { return [{ type: AudioBufferService, decorators: [{
                    type: Inject,
                    args: [AudioBufferService]
                }] }, { type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: AudioNode, decorators: [{
                    type: SkipSelf
                }, {
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }]; }, propDecorators: { bufferSetter: [{
                type: Input,
                args: ['buffer']
            }] } });

class WebAudioDelay extends DelayNode {
    constructor(context, node, modern, delayTimeArg, maxDelayTimeArg) {
        const delayTime = parse(delayTimeArg, 0);
        const maxDelayTime = parse(maxDelayTimeArg, 1);
        if (modern) {
            super(context, { delayTime, maxDelayTime });
            connect(node, this);
        }
        else {
            const result = context.createDelay(maxDelayTime);
            Object.setPrototypeOf(result, WebAudioDelay.prototype);
            connect(node, result);
            result.delayTime.value = delayTime;
            return result;
        }
    }
    ngOnDestroy() {
        this.disconnect();
    }
    static init(that, node) {
        connect(node, that);
    }
}
WebAudioDelay.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioDelay, deps: [{ token: AUDIO_CONTEXT }, { token: AUDIO_NODE, skipSelf: true }, { token: CONSTRUCTOR_SUPPORT }, { token: 'delayTime', attribute: true }, { token: 'maxDelayTime', attribute: true }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioDelay.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioDelay, selector: "[waDelayNode]", inputs: { channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation", delayTimeParam: ["delayTime", "delayTimeParam"] }, providers: [asAudioNode(WebAudioDelay)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
__decorate([
    audioParam('delayTime')
], WebAudioDelay.prototype, "delayTimeParam", void 0);
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioDelay, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waDelayNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [asAudioNode(WebAudioDelay)],
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: AudioNode, decorators: [{
                    type: SkipSelf
                }, {
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['delayTime']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['maxDelayTime']
                }] }]; }, propDecorators: { delayTimeParam: [{
                type: Input,
                args: ['delayTime']
            }] } });

class WebAudioDynamicsCompressor extends DynamicsCompressorNode {
    constructor(context, node, modern, attackArg, kneeArg, ratioArg, releaseArg, thresholdArg) {
        const attack = parse(attackArg, 0.003);
        const knee = parse(kneeArg, 30);
        const ratio = parse(ratioArg, 12);
        const release = parse(releaseArg, 0.25);
        const threshold = parse(thresholdArg, -24);
        if (modern) {
            super(context, { attack, knee, ratio, release, threshold });
            connect(node, this);
        }
        else {
            const result = context.createDynamicsCompressor();
            Object.setPrototypeOf(result, WebAudioDynamicsCompressor.prototype);
            connect(node, result);
            result.attack.value = attack;
            result.knee.value = knee;
            result.ratio.value = ratio;
            result.release.value = release;
            result.threshold.value = threshold;
            return result;
        }
    }
    ngOnDestroy() {
        this.disconnect();
    }
}
WebAudioDynamicsCompressor.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioDynamicsCompressor, deps: [{ token: AUDIO_CONTEXT }, { token: AUDIO_NODE, skipSelf: true }, { token: CONSTRUCTOR_SUPPORT }, { token: 'attack', attribute: true }, { token: 'knee', attribute: true }, { token: 'ratio', attribute: true }, { token: 'release', attribute: true }, { token: 'threshold', attribute: true }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioDynamicsCompressor.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioDynamicsCompressor, selector: "[waDynamicsCompressorNode]", inputs: { channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation", attackParam: ["attack", "attackParam"], kneeParam: ["knee", "kneeParam"], ratioParam: ["ratio", "ratioParam"], releaseParam: ["release", "releaseParam"], thresholdParam: ["threshold", "thresholdParam"] }, providers: [asAudioNode(WebAudioDynamicsCompressor)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
__decorate([
    audioParam('attack')
], WebAudioDynamicsCompressor.prototype, "attackParam", void 0);
__decorate([
    audioParam('knee')
], WebAudioDynamicsCompressor.prototype, "kneeParam", void 0);
__decorate([
    audioParam('ratio')
], WebAudioDynamicsCompressor.prototype, "ratioParam", void 0);
__decorate([
    audioParam('release')
], WebAudioDynamicsCompressor.prototype, "releaseParam", void 0);
__decorate([
    audioParam('threshold')
], WebAudioDynamicsCompressor.prototype, "thresholdParam", void 0);
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioDynamicsCompressor, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waDynamicsCompressorNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [asAudioNode(WebAudioDynamicsCompressor)],
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: AudioNode, decorators: [{
                    type: SkipSelf
                }, {
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['attack']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['knee']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['ratio']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['release']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['threshold']
                }] }]; }, propDecorators: { attackParam: [{
                type: Input,
                args: ['attack']
            }], kneeParam: [{
                type: Input,
                args: ['knee']
            }], ratioParam: [{
                type: Input,
                args: ['ratio']
            }], releaseParam: [{
                type: Input,
                args: ['release']
            }], thresholdParam: [{
                type: Input,
                args: ['threshold']
            }] } });

class WebAudioGain extends GainNode {
    constructor(context, node, modern, gainArg) {
        const gain = parse(gainArg, 1);
        if (modern) {
            super(context, { gain });
            connect(node, this);
        }
        else {
            const result = context.createGain();
            Object.setPrototypeOf(result, WebAudioGain.prototype);
            connect(node, result);
            result.gain.value = gain;
            return result;
        }
    }
    ngOnDestroy() {
        this.disconnect();
    }
}
WebAudioGain.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioGain, deps: [{ token: AUDIO_CONTEXT }, { token: AUDIO_NODE, skipSelf: true }, { token: CONSTRUCTOR_SUPPORT }, { token: 'gain', attribute: true }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioGain.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioGain, selector: "[waGainNode]", inputs: { channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation", gainParam: ["gain", "gainParam"] }, providers: [asAudioNode(WebAudioGain)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
__decorate([
    audioParam('gain')
], WebAudioGain.prototype, "gainParam", void 0);
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioGain, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waGainNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [asAudioNode(WebAudioGain)],
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: AudioNode, decorators: [{
                    type: SkipSelf
                }, {
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['gain']
                }] }]; }, propDecorators: { gainParam: [{
                type: Input,
                args: ['gain']
            }] } });

const FEEDBACK_COEFFICIENTS = new InjectionToken('A sequence of feedback coefficients for IIRFilterNode');

const FEEDFORWARD_COEFFICIENTS = new InjectionToken('A sequence of feedforward coefficients for IIRFilterNode');

class WebAudioIIRFilter extends IIRFilterNode {
    constructor(feedback, feedforward, context, modern, node) {
        if (modern) {
            super(context, { feedback, feedforward });
            connect(node, this);
        }
        else {
            const result = context.createIIRFilter(feedback, feedforward);
            Object.setPrototypeOf(result, WebAudioIIRFilter.prototype);
            connect(node, result);
            return result;
        }
    }
    ngOnDestroy() {
        this.disconnect();
    }
}
WebAudioIIRFilter.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioIIRFilter, deps: [{ token: FEEDBACK_COEFFICIENTS }, { token: FEEDFORWARD_COEFFICIENTS }, { token: AUDIO_CONTEXT }, { token: CONSTRUCTOR_SUPPORT }, { token: AUDIO_NODE, skipSelf: true }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioIIRFilter.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioIIRFilter, selector: "[waIIRFilterNode]", inputs: { channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation" }, providers: [asAudioNode(WebAudioIIRFilter)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioIIRFilter, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waIIRFilterNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [asAudioNode(WebAudioIIRFilter)],
                }]
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Inject,
                    args: [FEEDBACK_COEFFICIENTS]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [FEEDFORWARD_COEFFICIENTS]
                }] }, { type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }, { type: AudioNode, decorators: [{
                    type: SkipSelf
                }, {
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }]; } });

class WebAudioPanner extends PannerNode {
    constructor(context, node, modern) {
        if (modern) {
            super(context);
            connect(node, this);
        }
        else {
            const result = context.createPanner();
            Object.setPrototypeOf(result, WebAudioPanner.prototype);
            connect(node, result);
            return result;
        }
    }
    ngOnChanges() {
        if (this.positionX instanceof AudioParam) {
            return;
        }
        // Fallback for older browsers
        this.setOrientation(fallbackAudioParam(this.orientationXParam), fallbackAudioParam(this.orientationYParam), fallbackAudioParam(this.orientationZParam));
        this.setPosition(fallbackAudioParam(this.positionXParam), fallbackAudioParam(this.positionYParam), fallbackAudioParam(this.positionZParam));
    }
    ngOnDestroy() {
        this.disconnect();
    }
}
WebAudioPanner.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioPanner, deps: [{ token: AUDIO_CONTEXT }, { token: AUDIO_NODE, skipSelf: true }, { token: CONSTRUCTOR_SUPPORT }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioPanner.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioPanner, selector: "[waPannerNode]", inputs: { coneInnerAngle: "coneInnerAngle", coneOuterAngle: "coneOuterAngle", coneOuterGain: "coneOuterGain", distanceModel: "distanceModel", maxDistance: "maxDistance", panningModel: "panningModel", refDistance: "refDistance", rolloffFactor: "rolloffFactor", channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation", orientationXParam: "orientationXParam", orientationYParam: "orientationYParam", orientationZParam: "orientationZParam", positionXParam: "positionXParam", positionYParam: "positionYParam", positionZParam: "positionZParam" }, providers: [asAudioNode(WebAudioPanner)], exportAs: ["AudioNode"], usesInheritance: true, usesOnChanges: true, ngImport: i0 });
__decorate([
    audioParam('orientationX')
], WebAudioPanner.prototype, "orientationXParam", void 0);
__decorate([
    audioParam('orientationY')
], WebAudioPanner.prototype, "orientationYParam", void 0);
__decorate([
    audioParam('orientationZ')
], WebAudioPanner.prototype, "orientationZParam", void 0);
__decorate([
    audioParam('positionX')
], WebAudioPanner.prototype, "positionXParam", void 0);
__decorate([
    audioParam('positionY')
], WebAudioPanner.prototype, "positionYParam", void 0);
__decorate([
    audioParam('positionZ')
], WebAudioPanner.prototype, "positionZParam", void 0);
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioPanner, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waPannerNode]',
                    exportAs: 'AudioNode',
                    inputs: [
                        'coneInnerAngle',
                        'coneOuterAngle',
                        'coneOuterGain',
                        'distanceModel',
                        'maxDistance',
                        'panningModel',
                        'refDistance',
                        'rolloffFactor',
                        'channelCount',
                        'channelCountMode',
                        'channelInterpretation',
                    ],
                    providers: [asAudioNode(WebAudioPanner)],
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: AudioNode, decorators: [{
                    type: SkipSelf
                }, {
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }]; }, propDecorators: { orientationXParam: [{
                type: Input
            }], orientationYParam: [{
                type: Input
            }], orientationZParam: [{
                type: Input
            }], positionXParam: [{
                type: Input
            }], positionYParam: [{
                type: Input
            }], positionZParam: [{
                type: Input
            }] } });

class WebAudioScriptProcessor extends ScriptProcessorNode {
    constructor(bufferSize, numberOfInputChannels, numberOfOutputChannels, context, node) {
        try {
            const result = context.createScriptProcessor(parseInt(bufferSize || '', 10) || 0, parseInt(numberOfInputChannels || '', 10) || 2, parseInt(numberOfOutputChannels || '', 10) || 2);
            Object.setPrototypeOf(result, WebAudioScriptProcessor.prototype);
            connect(node, result);
            const audioprocess = new EventEmitter();
            result.audioprocess = audioprocess;
            result.onaudioprocess = e => audioprocess.emit(e);
            return result;
        }
        catch (_) {
            // noop
        }
        super();
    }
    ngOnDestroy() {
        this.disconnect();
    }
}
WebAudioScriptProcessor.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioScriptProcessor, deps: [{ token: 'bufferSize', attribute: true }, { token: 'numberOfInputChannels', attribute: true }, { token: 'numberOfOutputChannels', attribute: true }, { token: AUDIO_CONTEXT }, { token: AUDIO_NODE, skipSelf: true }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioScriptProcessor.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioScriptProcessor, selector: "[waScriptProcessorNode]", inputs: { channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation" }, outputs: { audioprocess: "audioprocess" }, providers: [asAudioNode(WebAudioScriptProcessor)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioScriptProcessor, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waScriptProcessorNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [asAudioNode(WebAudioScriptProcessor)],
                }]
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Attribute,
                    args: ['bufferSize']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['numberOfInputChannels']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['numberOfOutputChannels']
                }] }, { type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: AudioNode, decorators: [{
                    type: SkipSelf
                }, {
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }]; }, propDecorators: { audioprocess: [{
                type: Output
            }] } });

class WebAudioStereoPanner extends StereoPannerNode {
    constructor(context, node, panArg) {
        const pan = parse(panArg, 0);
        try {
            new StereoPannerNode(context);
        }
        catch (_a) {
            const result = context.createPanner();
            Object.setPrototypeOf(result, WebAudioStereoPanner.prototype);
            result.fallbackToPannerNode(fallbackAudioParam(pan));
            connect(node, result);
            return result;
        }
        super(context, { pan });
        connect(node, this);
    }
    set panParam(pan) {
        if ('setPosition' in this) {
            /** fallback for browsers not supporting {@link StereoPannerNode} */
            // @ts-ignore
            this.fallbackToPannerNode(fallbackAudioParam(pan));
        }
        else {
            processAudioParam(this.pan, pan, this.context.currentTime);
        }
    }
    ngOnDestroy() {
        this.disconnect();
    }
    // @ts-ignore
    fallbackToPannerNode(pan) {
        const xDeg = pan * 100;
        const zDeg = xDeg > 0 ? 270 - xDeg : xDeg + 90;
        const x = Math.sin(xDeg * (Math.PI / 180));
        const z = Math.sin(zDeg * (Math.PI / 180));
        // @ts-ignore
        this.setPosition(x, 0, z);
    }
}
WebAudioStereoPanner.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioStereoPanner, deps: [{ token: AUDIO_CONTEXT }, { token: AUDIO_NODE, skipSelf: true }, { token: 'pan', attribute: true }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioStereoPanner.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioStereoPanner, selector: "[waStereoPannerNode]", inputs: { channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation", panParam: ["pan", "panParam"] }, providers: [asAudioNode(WebAudioStereoPanner)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioStereoPanner, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waStereoPannerNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [asAudioNode(WebAudioStereoPanner)],
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: AudioNode, decorators: [{
                    type: SkipSelf
                }, {
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['pan']
                }] }]; }, propDecorators: { panParam: [{
                type: Input,
                args: ['pan']
            }] } });

class WebAudioWaveShaper extends WaveShaperNode {
    constructor(context, node, modern) {
        if (modern) {
            super(context);
            connect(node, this);
        }
        else {
            const result = context.createWaveShaper();
            Object.setPrototypeOf(result, WebAudioWaveShaper.prototype);
            connect(node, result);
            return result;
        }
    }
    ngOnDestroy() {
        this.disconnect();
    }
}
WebAudioWaveShaper.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioWaveShaper, deps: [{ token: AUDIO_CONTEXT }, { token: AUDIO_NODE, skipSelf: true }, { token: CONSTRUCTOR_SUPPORT }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioWaveShaper.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioWaveShaper, selector: "[waWaveShaperNode]", inputs: { oversample: "oversample", curve: "curve", channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation" }, providers: [asAudioNode(WebAudioWaveShaper)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioWaveShaper, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waWaveShaperNode]',
                    exportAs: 'AudioNode',
                    inputs: [
                        'oversample',
                        'curve',
                        'channelCount',
                        'channelCountMode',
                        'channelInterpretation',
                    ],
                    providers: [asAudioNode(WebAudioWaveShaper)],
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: AudioNode, decorators: [{
                    type: SkipSelf
                }, {
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }]; } });

class WebAudioWorklet extends AudioWorkletNode {
    constructor(context, node, name) {
        super(context, name);
        this.processorerror = new EventEmitter();
        this.onprocessorerror = () => this.processorerror.emit();
        connect(node, this);
    }
    ngOnDestroy() {
        this.disconnect();
    }
}
WebAudioWorklet.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioWorklet, deps: [{ token: AUDIO_CONTEXT }, { token: AUDIO_NODE, skipSelf: true }, { token: 'name', attribute: true }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioWorklet.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioWorklet, selector: "[waAudioWorkletNode][name]", inputs: { channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation" }, outputs: { processorerror: "processorerror" }, providers: [asAudioNode(WebAudioWorklet)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioWorklet, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waAudioWorkletNode][name]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [asAudioNode(WebAudioWorklet)],
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: AudioNode, decorators: [{
                    type: SkipSelf
                }, {
                    type: Inject,
                    args: [AUDIO_NODE]
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['name']
                }] }]; }, propDecorators: { processorerror: [{
                type: Output
            }] } });

class WebAudioParamPipe {
    /**
     * Creates {@link AudioParamAutomation} to use with {@link AudioParam} inputs
     *
     * @param value target value
     * @param duration duration of the automation
     * @param mode either instant for given duration or linear/exponential ramp
     */
    transform(value, duration, mode = 'exponential') {
        return value instanceof Array
            ? {
                value,
                duration,
            }
            : {
                value,
                duration,
                mode,
            };
    }
}
WebAudioParamPipe.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioParamPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
WebAudioParamPipe.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioParamPipe, name: "waAudioParam" });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioParamPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'waAudioParam',
                }]
        }] });

class WebAudioPeriodicWavePipe {
    constructor(context) {
        this.context = context;
    }
    /**
     * Creates {@link PeriodicWave} to use with {@link OscillatorNode}
     *
     * @param real cosine terms (traditionally the A terms)
     * @param imag sine terms (traditionally the B terms)
     * @param disableNormalization see {@lin PeriodicWaveConstraints}
     */
    transform(real, imag, disableNormalization) {
        return this.context.createPeriodicWave(new Float32Array(real), imag ? new Float32Array(imag) : new Float32Array(real.length), {
            disableNormalization: !!disableNormalization,
        });
    }
}
WebAudioPeriodicWavePipe.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioPeriodicWavePipe, deps: [{ token: AUDIO_CONTEXT }], target: i0.ɵɵFactoryTarget.Pipe });
WebAudioPeriodicWavePipe.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioPeriodicWavePipe, name: "waPeriodicWave" });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioPeriodicWavePipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'waPeriodicWave',
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }]; } });

class WebAudioBufferSource extends AudioBufferSourceNode {
    constructor(audioBufferService, context, modern, autoplay, detuneArg, playbackRateArg) {
        const detune = parse(detuneArg, 0);
        const playbackRate = parse(playbackRateArg, 1);
        if (modern) {
            super(context, { detune, playbackRate });
            WebAudioBufferSource.init(this, null, autoplay, audioBufferService);
        }
        else {
            const result = context.createBufferSource();
            Object.setPrototypeOf(WebAudioBufferSource.prototype, Object.getPrototypeOf(result));
            Object.setPrototypeOf(result, WebAudioBufferSource.prototype);
            result.playbackRate.value = playbackRate;
            WebAudioBufferSource.init(result, null, autoplay, audioBufferService);
            return result;
        }
    }
    set bufferSetter(source) {
        this.buffer$.next(source);
    }
    ngOnDestroy() {
        this.buffer$.complete();
        try {
            this.stop();
        }
        catch (_a) {
            // noop
        }
        this.disconnect();
    }
    static init(that, _node, autoplay, audioBufferService) {
        if (autoplay !== null) {
            that.start();
        }
        const ended = new EventEmitter();
        that.ended = ended;
        that.onended = () => ended.emit();
        that.buffer$ = new Subject();
        that.buffer$
            .pipe(switchMap(source => typeof source === 'string'
            ? audioBufferService.fetch(source)
            : of(source)))
            .subscribe(buffer => {
            that.buffer = buffer;
        });
    }
}
WebAudioBufferSource.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioBufferSource, deps: [{ token: AudioBufferService }, { token: AUDIO_CONTEXT }, { token: CONSTRUCTOR_SUPPORT }, { token: 'autoplay', attribute: true }, { token: 'detune', attribute: true }, { token: 'playbackRate', attribute: true }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioBufferSource.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioBufferSource, selector: "[waAudioBufferSourceNode]", inputs: { loop: "loop", loopStart: "loopStart", loopEnd: "loopEnd", channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation", bufferSetter: ["buffer", "bufferSetter"], detuneParam: ["detune", "detuneParam"], playbackRateParam: ["playbackRate", "playbackRateParam"] }, outputs: { ended: "ended" }, providers: [asAudioNode(WebAudioBufferSource)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
__decorate([
    audioParam('detune')
], WebAudioBufferSource.prototype, "detuneParam", void 0);
__decorate([
    audioParam('playbackRate')
], WebAudioBufferSource.prototype, "playbackRateParam", void 0);
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioBufferSource, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waAudioBufferSourceNode]',
                    exportAs: 'AudioNode',
                    inputs: [
                        'loop',
                        'loopStart',
                        'loopEnd',
                        'channelCount',
                        'channelCountMode',
                        'channelInterpretation',
                    ],
                    providers: [asAudioNode(WebAudioBufferSource)],
                }]
        }], ctorParameters: function () { return [{ type: AudioBufferService, decorators: [{
                    type: Inject,
                    args: [AudioBufferService]
                }] }, { type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['autoplay']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['detune']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['playbackRate']
                }] }]; }, propDecorators: { bufferSetter: [{
                type: Input,
                args: ['buffer']
            }], detuneParam: [{
                type: Input,
                args: ['detune']
            }], playbackRateParam: [{
                type: Input,
                args: ['playbackRate']
            }], ended: [{
                type: Output
            }] } });

class WebAudioConstantSource extends ConstantSourceNode {
    constructor(context, autoplay, offset) {
        super(context, {
            offset: parse(offset, 0),
        });
        this.ended = new EventEmitter();
        this.onended = () => this.ended.emit();
        if (autoplay !== null) {
            this.start();
        }
    }
    ngOnDestroy() {
        try {
            this.stop();
        }
        catch (_a) {
            // noop
        }
        this.disconnect();
    }
}
WebAudioConstantSource.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioConstantSource, deps: [{ token: AUDIO_CONTEXT }, { token: 'autoplay', attribute: true }, { token: 'offset', attribute: true }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioConstantSource.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioConstantSource, selector: "[waConstantSourceNode]", inputs: { channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation", offsetParam: ["offset", "offsetParam"] }, outputs: { ended: "ended" }, providers: [asAudioNode(WebAudioConstantSource)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
__decorate([
    audioParam('offset')
], WebAudioConstantSource.prototype, "offsetParam", void 0);
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioConstantSource, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waConstantSourceNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [asAudioNode(WebAudioConstantSource)],
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['autoplay']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['offset']
                }] }]; }, propDecorators: { offsetParam: [{
                type: Input,
                args: ['offset']
            }], ended: [{
                type: Output
            }] } });

class WebAudioMediaSource extends MediaElementAudioSourceNode {
    constructor(context, modern, { nativeElement }) {
        if (modern) {
            super(context, { mediaElement: nativeElement });
        }
        else {
            const result = context.createMediaElementSource(nativeElement);
            Object.setPrototypeOf(result, WebAudioMediaSource.prototype);
            return result;
        }
    }
    ngOnDestroy() {
        this.disconnect();
    }
}
WebAudioMediaSource.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioMediaSource, deps: [{ token: AUDIO_CONTEXT }, { token: CONSTRUCTOR_SUPPORT }, { token: ElementRef }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioMediaSource.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioMediaSource, selector: "audio[waMediaElementAudioSourceNode], video[waMediaElementAudioSourceNode]", providers: [asAudioNode(WebAudioMediaSource)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioMediaSource, decorators: [{
            type: Directive,
            args: [{
                    selector: 'audio[waMediaElementAudioSourceNode], video[waMediaElementAudioSourceNode]',
                    exportAs: 'AudioNode',
                    providers: [asAudioNode(WebAudioMediaSource)],
                }]
        }], ctorParameters: function () { return [{ type: AudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }, { type: i0.ElementRef, decorators: [{
                    type: Inject,
                    args: [ElementRef]
                }] }]; } });

const MEDIA_STREAM = new InjectionToken('MediaStream for MediaStreamAudioSourceNode');

class WebAudioMediaStreamSource extends MediaStreamAudioSourceNode {
    constructor(mediaStream, context, modern) {
        if (modern) {
            super(context, { mediaStream });
        }
        else {
            const result = context.createMediaStreamSource(mediaStream);
            Object.setPrototypeOf(result, WebAudioMediaStreamSource.prototype);
            return result;
        }
    }
    ngOnDestroy() {
        this.disconnect();
    }
}
WebAudioMediaStreamSource.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioMediaStreamSource, deps: [{ token: MEDIA_STREAM }, { token: AUDIO_CONTEXT }, { token: CONSTRUCTOR_SUPPORT }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioMediaStreamSource.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioMediaStreamSource, selector: "[waMediaStreamAudioSourceNode]", providers: [asAudioNode(WebAudioMediaStreamSource)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioMediaStreamSource, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waMediaStreamAudioSourceNode]',
                    exportAs: 'AudioNode',
                    providers: [asAudioNode(WebAudioMediaStreamSource)],
                }]
        }], ctorParameters: function () { return [{ type: MediaStream, decorators: [{
                    type: Inject,
                    args: [MEDIA_STREAM]
                }] }, { type: AudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }]; } });

class WebAudioOscillator extends OscillatorNode {
    constructor(context, modern, autoplay, detuneArg, frequencyArg) {
        const detune = parse(detuneArg, 0);
        const frequency = parse(frequencyArg, 440);
        if (modern) {
            super(context, { detune, frequency });
            WebAudioOscillator.init(this, null, autoplay);
        }
        else {
            const result = context.createOscillator();
            Object.setPrototypeOf(WebAudioOscillator.prototype, Object.getPrototypeOf(result));
            Object.setPrototypeOf(result, WebAudioOscillator.prototype);
            result.detune.value = detune;
            result.frequency.value = frequency;
            WebAudioOscillator.init(result, null, autoplay);
            return result;
        }
    }
    set periodicWave(periodicWave) {
        this.setPeriodicWave(periodicWave);
    }
    ngOnDestroy() {
        try {
            this.stop();
        }
        catch (_a) {
            // noop
        }
        this.disconnect();
    }
    static init(that, node, autoplay) {
        connect(node, that);
        if (autoplay !== null) {
            that.start();
        }
        const ended = new EventEmitter();
        that.ended = ended;
        that.onended = () => ended.emit();
    }
}
WebAudioOscillator.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioOscillator, deps: [{ token: AUDIO_CONTEXT }, { token: CONSTRUCTOR_SUPPORT }, { token: 'autoplay', attribute: true }, { token: 'detune', attribute: true }, { token: 'frequency', attribute: true }], target: i0.ɵɵFactoryTarget.Directive });
WebAudioOscillator.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.15", type: WebAudioOscillator, selector: "[waOscillatorNode]", inputs: { type: "type", channelCount: "channelCount", channelCountMode: "channelCountMode", channelInterpretation: "channelInterpretation", periodicWave: "periodicWave", detuneParam: ["detune", "detuneParam"], frequencyParam: ["frequency", "frequencyParam"] }, outputs: { ended: "ended" }, providers: [asAudioNode(WebAudioOscillator)], exportAs: ["AudioNode"], usesInheritance: true, ngImport: i0 });
__decorate([
    audioParam('detune')
], WebAudioOscillator.prototype, "detuneParam", void 0);
__decorate([
    audioParam('frequency')
], WebAudioOscillator.prototype, "frequencyParam", void 0);
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioOscillator, decorators: [{
            type: Directive,
            args: [{
                    selector: '[waOscillatorNode]',
                    exportAs: 'AudioNode',
                    inputs: ['type', 'channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [asAudioNode(WebAudioOscillator)],
                }]
        }], ctorParameters: function () { return [{ type: BaseAudioContext, decorators: [{
                    type: Inject,
                    args: [AUDIO_CONTEXT]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONSTRUCTOR_SUPPORT]
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['autoplay']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['detune']
                }] }, { type: undefined, decorators: [{
                    type: Attribute,
                    args: ['frequency']
                }] }]; }, propDecorators: { periodicWave: [{
                type: Input
            }], detuneParam: [{
                type: Input,
                args: ['detune']
            }], frequencyParam: [{
                type: Input,
                args: ['frequency']
            }], ended: [{
                type: Output
            }] } });

const AUDIO_WORKLET_PROCESSORS = new InjectionToken('Web Audio API worklet processors paths', {
    providedIn: 'root',
    factory: () => [],
});

const AUDIO_WORKLET_SUPPORT = new InjectionToken('AudioWorklet browser support', {
    factory: () => !!inject(AUDIO_CONTEXT).audioWorklet,
});

const AUDIO_WORKLET_PROCESSORS_READY = new InjectionToken('Web Audio API worklet processors resolution promise', {
    providedIn: 'root',
    factory: () => {
        const context = inject(AUDIO_CONTEXT);
        const processors = inject(AUDIO_WORKLET_PROCESSORS);
        if (!inject(AUDIO_WORKLET_SUPPORT)) {
            return Promise.reject('AudioWorklet is not supported').catch(() => false);
        }
        return Promise.all(processors.map(processor => context.audioWorklet.addModule(processor))).then(() => true);
    },
});

const WEB_AUDIO_SUPPORT = new InjectionToken('Web Audio API support', {
    providedIn: 'root',
    factory: () => !!AudioContext,
});

class WebAudioModule {
}
WebAudioModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
WebAudioModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioModule, declarations: [WebAudioContext,
        WebAudioChannel,
        WebAudioDestination,
        WebAudioListener,
        WebAudioOfflineContext,
        WebAudioOutput,
        WebAudioMediaStreamDestination,
        WebAudioBufferSource,
        WebAudioConstantSource,
        WebAudioMediaSource,
        WebAudioMediaStreamSource,
        WebAudioOscillator,
        WebAudioAnalyser,
        WebAudioBiquadFilter,
        WebAudioChannelSplitter,
        WebAudioChannelMerger,
        WebAudioConvolver,
        WebAudioDelay,
        WebAudioDynamicsCompressor,
        WebAudioGain,
        WebAudioIIRFilter,
        WebAudioPanner,
        WebAudioScriptProcessor,
        WebAudioStereoPanner,
        WebAudioWaveShaper,
        WebAudioWorklet,
        WebAudioParamPipe,
        WebAudioPeriodicWavePipe], exports: [WebAudioContext,
        WebAudioChannel,
        WebAudioDestination,
        WebAudioListener,
        WebAudioOfflineContext,
        WebAudioOutput,
        WebAudioMediaStreamDestination,
        WebAudioBufferSource,
        WebAudioConstantSource,
        WebAudioMediaSource,
        WebAudioMediaStreamSource,
        WebAudioOscillator,
        WebAudioAnalyser,
        WebAudioBiquadFilter,
        WebAudioChannelSplitter,
        WebAudioChannelMerger,
        WebAudioConvolver,
        WebAudioDelay,
        WebAudioDynamicsCompressor,
        WebAudioGain,
        WebAudioIIRFilter,
        WebAudioPanner,
        WebAudioScriptProcessor,
        WebAudioStereoPanner,
        WebAudioWaveShaper,
        WebAudioWorklet,
        WebAudioParamPipe,
        WebAudioPeriodicWavePipe] });
WebAudioModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0, type: WebAudioModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        WebAudioContext,
                        WebAudioChannel,
                        WebAudioDestination,
                        WebAudioListener,
                        WebAudioOfflineContext,
                        WebAudioOutput,
                        WebAudioMediaStreamDestination,
                        WebAudioBufferSource,
                        WebAudioConstantSource,
                        WebAudioMediaSource,
                        WebAudioMediaStreamSource,
                        WebAudioOscillator,
                        WebAudioAnalyser,
                        WebAudioBiquadFilter,
                        WebAudioChannelSplitter,
                        WebAudioChannelMerger,
                        WebAudioConvolver,
                        WebAudioDelay,
                        WebAudioDynamicsCompressor,
                        WebAudioGain,
                        WebAudioIIRFilter,
                        WebAudioPanner,
                        WebAudioScriptProcessor,
                        WebAudioStereoPanner,
                        WebAudioWaveShaper,
                        WebAudioWorklet,
                        WebAudioParamPipe,
                        WebAudioPeriodicWavePipe,
                    ],
                    exports: [
                        WebAudioContext,
                        WebAudioChannel,
                        WebAudioDestination,
                        WebAudioListener,
                        WebAudioOfflineContext,
                        WebAudioOutput,
                        WebAudioMediaStreamDestination,
                        WebAudioBufferSource,
                        WebAudioConstantSource,
                        WebAudioMediaSource,
                        WebAudioMediaStreamSource,
                        WebAudioOscillator,
                        WebAudioAnalyser,
                        WebAudioBiquadFilter,
                        WebAudioChannelSplitter,
                        WebAudioChannelMerger,
                        WebAudioConvolver,
                        WebAudioDelay,
                        WebAudioDynamicsCompressor,
                        WebAudioGain,
                        WebAudioIIRFilter,
                        WebAudioPanner,
                        WebAudioScriptProcessor,
                        WebAudioStereoPanner,
                        WebAudioWaveShaper,
                        WebAudioWorklet,
                        WebAudioParamPipe,
                        WebAudioPeriodicWavePipe,
                    ],
                }]
        }] });

/**
 * Public API Surface of @ng-web-apis/audio
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AUDIO_CONTEXT, AUDIO_NODE, AUDIO_WORKLET_PROCESSORS, AUDIO_WORKLET_PROCESSORS_READY, AUDIO_WORKLET_SUPPORT, AudioBufferService, FEEDBACK_COEFFICIENTS, FEEDFORWARD_COEFFICIENTS, MEDIA_STREAM, WEB_AUDIO_SUPPORT, WebAudioAnalyser, WebAudioBiquadFilter, WebAudioBufferSource, WebAudioChannel, WebAudioChannelMerger, WebAudioChannelSplitter, WebAudioConstantSource, WebAudioContext, WebAudioConvolver, WebAudioDelay, WebAudioDestination, WebAudioDynamicsCompressor, WebAudioGain, WebAudioIIRFilter, WebAudioListener, WebAudioMediaSource, WebAudioMediaStreamDestination, WebAudioMediaStreamSource, WebAudioModule, WebAudioOfflineContext, WebAudioOscillator, WebAudioOutput, WebAudioPanner, WebAudioParamPipe, WebAudioPeriodicWavePipe, WebAudioScriptProcessor, WebAudioStereoPanner, WebAudioWaveShaper, WebAudioWorklet, asAudioNode, audioParam };
//# sourceMappingURL=ng-web-apis-audio.js.map
